package com.mobage.nbpf.sampleapp.auth;

import com.mobage.nbpf.sampleapp.SessionStorage;
import com.mobage.nbpf.sampleapp.config.SampleAppConfig;
import com.mobage.nbpf.sampleapp.payment.MobagePaymentService;

import java.util.Map;
import java.util.UUID;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.util.UriComponentsBuilder;

@Controller
@RequestMapping("/mobage/auth")
public class MobageAuthController {
  private SampleAppConfig config;
  private SessionStorage session;
  Logger logger = LoggerFactory.getLogger(MobagePaymentService.class);

  @Autowired
  public MobageAuthController(SampleAppConfig config, SessionStorage session) {
    this.config = config;
    this.session = session;
  }

  /** request handler for web page for Sign-in with Mobage. */
  @GetMapping
  public String index(
      @RequestParam Map<String, String> params,
      Model model,
      HttpServletResponse response) {

    if (session.getState() == null) {
      // Set state parameter in HttpSession
      // state parameter is needed for preventing CSRF
      session.setState(UUID.randomUUID().toString());
    }

    model.addAttribute("clientId", config.getClientId());
    model.addAttribute("redirectUri", config.getRedirectUri());
    model.addAttribute("jssdkPath", config.getJssdkPath());
    model.addAttribute("state", session.getState());
    model.addAttribute("sessionState", session.getSessionState());

    return "auth";
  }

  @GetMapping("/login")
  public void login(HttpServletResponse response) throws IOException {

    if (session.getSessionState() == null
        || session.getUserId() == null
        || session.getAccessToken() == null) {

      String state = UUID.randomUUID().toString();
      session.setState(state);

      UriComponentsBuilder builder =
          UriComponentsBuilder.fromUriString(config.getAuthorizeEndpoint());
      builder.queryParam("client_id", config.getClientId());
      builder.queryParam("redirect_uri", config.getRedirectUri());
      builder.queryParam("response_type", "code");
      builder.queryParam("scope", "openid common_api");
      builder.queryParam("state", state);

      response.sendRedirect(builder.toUriString());
      return;
    }
    logger.info("already logged in");
    response.sendRedirect("/mobage/auth");
  }

  @GetMapping("/logout")
  public void logout(HttpServletRequest request, HttpServletResponse response) throws IOException {
    request.getSession().invalidate();
    response.sendRedirect("/");
  }
}
